package splg.examples.onlinegrouper;

import splg.definition.splgcatalog.SplgCatalog;
import splg.data.falldaten.Falldaten;
import splg.data.falldaten.FalldatenBehandlung;
import splg.data.falldaten.FalldatenDiagnose;
import splg.data.fallzuteilung.Bereich;
import splg.grouper.Grouper;
import splg.output.gaf.GafBuilder;
import splg.output.grouper.GrouperOutput;

import javax.swing.*;
import java.awt.*;
import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * This is a very simple examples application that demonstrates
 * how to integrate the SPLG Kernel into an application.
 *
 * The example uses Java Swing technology to demonstrate a simple
 * and limited online grouper application.
 *
 * You can select the definition version, enter the case data
 * and group the case. The output shows the group result as well
 * as the GAF file contents.
 *
 * This grouper code is limited in several ways and only useful
 * for demonstration purposes:
 * - Maximal 10 diagnoses
 * - Maximal 10 treatments
 * - Only CHOP/ICD codes used, no laterality or other secondary information
 * - No possibility to code surgeon information for MFZO purposes
 * - Minimal error handling
 *
 * Requirements:
 *
 * This tool expects in the current directory the existence of the
 * 4_lib directory with all appropriate sub-directories. Typically
 * this structure looks like this:
 * 4_lib
 *   Akut
 *     2012
 *     ...
 *     2019
 *     2020
 * The year-folders (e.g. 2019) must contain the splg definition and
 * the spitallisten files appropriate for this year. Normally, you
 * just use the folder as provided with the splg grouper download.
 *
 * For demonstration purposes, the tool contains definintions for
 * a single year (2020, Akut).
 *
 * ****************************************************************
 * YOU NEED A VALID LICENSE FILE in order for this example to work!
 * ****************************************************************
 */
public class OnlineGrouperExample extends JFrame {

    private static final long serialVersionUID = 1L;

    private JComboBox<String> bereich = new JComboBox<>();
    private JComboBox<String> defversion = new JComboBox<>();

    private JTextField burnr = new JTextField(10);
    private JTextField plz = new JTextField(10);
    private JTextField fallid = new JTextField(10);
    private JTextField agey = new JTextField(10);
    private JTextField aged = new JTextField(10);
    private JTextField ggw = new JTextField(10);
    private JTextField ssw = new JTextField(10);
    private JTextField dmb = new JTextField(10);

    private JTextField diagnose1 = new JTextField(10);
    private JTextField diagnose2 = new JTextField(10);
    private JTextField diagnose3 = new JTextField(10);
    private JTextField diagnose4 = new JTextField(10);
    private JTextField diagnose5 = new JTextField(10);
    private JTextField diagnose6 = new JTextField(10);
    private JTextField diagnose7 = new JTextField(10);
    private JTextField diagnose8 = new JTextField(10);
    private JTextField diagnose9 = new JTextField(10);
    private JTextField diagnose10 = new JTextField(10);

    private JTextField behandlung1 = new JTextField(10);
    private JTextField behandlung2 = new JTextField(10);
    private JTextField behandlung3 = new JTextField(10);
    private JTextField behandlung4 = new JTextField(10);
    private JTextField behandlung5 = new JTextField(10);
    private JTextField behandlung6 = new JTextField(10);
    private JTextField behandlung7 = new JTextField(10);
    private JTextField behandlung8 = new JTextField(10);
    private JTextField behandlung9 = new JTextField(10);
    private JTextField behandlung10 = new JTextField(10);

    private JTextArea output = new JTextArea();

    public OnlineGrouperExample() {
        super("SPLG Online Grouper Example");
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        getContentPane().setLayout(new BorderLayout());
        setSize(700, 800);
        setLocationRelativeTo(null);

        JPanel form = new JPanel();
        form.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
        form.setLayout(new GridBagLayout());

        int x = 0;
        int y = 0;

        addField(form, x, y++, "Bereich", bereich);
        addField(form, x, y++, "Defversion", defversion);
        addField(form, x, y++, "BUR-Nr", burnr);
        addField(form, x, y++, "PLZ", plz);
        addField(form, x, y++, "Fall-ID", fallid);
        addField(form, x, y++, "Alter (Jahre)", agey);
        addField(form, x, y++, "Alter (Tage)", aged);
        addField(form, x, y++, "GGW", ggw);
        addField(form, x, y++, "SSW", ssw);
        addField(form, x, y++, "DMB", dmb);

        x += 2;
        y = 0;

        addField(form, x, y++, "Diagnose1", diagnose1);
        addField(form, x, y++, "Diagnose2", diagnose2);
        addField(form, x, y++, "Diagnose3", diagnose3);
        addField(form, x, y++, "Diagnose4", diagnose4);
        addField(form, x, y++, "Diagnose5", diagnose5);
        addField(form, x, y++, "Diagnose6", diagnose6);
        addField(form, x, y++, "Diagnose7", diagnose7);
        addField(form, x, y++, "Diagnose8", diagnose8);
        addField(form, x, y++, "Diagnose9", diagnose9);
        addField(form, x, y++, "Diagnose10", diagnose10);

        x += 2;
        y = 0;

        addField(form, x, y++, "Behandlung1", behandlung1);
        addField(form, x, y++, "Behandlung2", behandlung2);
        addField(form, x, y++, "Behandlung3", behandlung3);
        addField(form, x, y++, "Behandlung4", behandlung4);
        addField(form, x, y++, "Behandlung5", behandlung5);
        addField(form, x, y++, "Behandlung6", behandlung6);
        addField(form, x, y++, "Behandlung7", behandlung7);
        addField(form, x, y++, "Behandlung8", behandlung8);
        addField(form, x, y++, "Behandlung9", behandlung9);
        addField(form, x, y++, "Behandlung10", behandlung10);

        JButton group = new JButton("Gruppieren");
        addField(form, 0, y, null, group);
        group.addActionListener((e) -> group());

        updateBereiche(bereich);
        updateDefversions(defversion, bereich);

        bereich.addActionListener((e) -> updateDefversions(defversion, bereich));

        getContentPane().add(form, BorderLayout.NORTH);
        getContentPane().add(new JScrollPane(output, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED), BorderLayout.CENTER);
    }

    private void group() {
        this.output.setText("");

        /**
         * Normally, one would allocate a onlinegrouper object and then
         * group multiple cases with it. Here, we reallocate a
         * onlinegrouper each time we group a case. Since each grouping process
         * is triggered by the user (clicking the "group" button),
         * this is not a performance issue.
         */

        File dir = new File(new File(getBaseDir(), (String) bereich.getSelectedItem()), (String) defversion.getSelectedItem());

        try (Grouper grouper = Grouper.create(getBereich(bereich), dir)) {

            Falldaten fall = new Falldaten();

            fall.burnr = burnr.getText();
            fall.plz = burnr.getText();
            fall.fallid = burnr.getText();
            fall.agey = agey.getText();
            fall.aged = aged.getText();
            fall.ggw = ggw.getText();
            fall.ssw = ssw.getText();
            fall.dmb = dmb.getText();

            addDiagnose(fall, 0, diagnose1.getText());
            addDiagnose(fall, 1, diagnose2.getText());
            addDiagnose(fall, 2, diagnose3.getText());
            addDiagnose(fall, 3, diagnose4.getText());
            addDiagnose(fall, 4, diagnose5.getText());
            addDiagnose(fall, 5, diagnose6.getText());
            addDiagnose(fall, 6, diagnose7.getText());
            addDiagnose(fall, 7, diagnose8.getText());
            addDiagnose(fall, 8, diagnose9.getText());
            addDiagnose(fall, 9, diagnose10.getText());

            addBehandlung(fall, 0, behandlung1.getText());
            addBehandlung(fall, 1, behandlung2.getText());
            addBehandlung(fall, 2, behandlung3.getText());
            addBehandlung(fall, 3, behandlung4.getText());
            addBehandlung(fall, 4, behandlung5.getText());
            addBehandlung(fall, 5, behandlung6.getText());
            addBehandlung(fall, 6, behandlung7.getText());
            addBehandlung(fall, 7, behandlung8.getText());
            addBehandlung(fall, 8, behandlung9.getText());
            addBehandlung(fall, 9, behandlung10.getText());

            GrouperOutput groupresult = grouper.group(fall);

            StringBuilder result = new StringBuilder();
            result.append(groupresult.toJSON());
            result.append("\r\n");

            result.append("\r\n");

            File catfile = null;
            for (File file : dir.listFiles()) {
                if (file.getName().startsWith("catalog_")) {
                    catfile = file;
                    break;
                }
            }
            GafBuilder gafbuilder = GafBuilder.getBasicBuilder(SplgCatalog.load(catfile));
            result.append(gafbuilder.getHeader()).append("\r\n");
            for (String line : gafbuilder.getLines("---", fall, groupresult)) {
                result.append(line).append("\r\n");
            }

            this.output.setText(result.toString());
        } catch (Exception e) {
            this.output.setText(e.toString());
        }
    }

    private void addDiagnose(Falldaten fall, int rang, String diagnose) {
        FalldatenDiagnose falldiagnose = new FalldatenDiagnose();
        falldiagnose.rang = rang;
        falldiagnose.code = diagnose;
        fall.diagnosen.add(falldiagnose);
    }

    private void addBehandlung(Falldaten fall, int rang, String behandlung) {
        FalldatenBehandlung fallbehandlung = new FalldatenBehandlung();
        fallbehandlung.rang = rang;
        fallbehandlung.code = behandlung;
        fall.behandlungen.add(fallbehandlung);
    }

    private File getBaseDir() {
        File base = new File("07_Definitions/Release"); // internal file, ignore outside of GD
        if (base.exists()) return base;
        return new File("4_lib");
    }

    private void updateBereiche(JComboBox<String> bereich) {
        String oldselection = (String) bereich.getSelectedItem();
        bereich.removeAllItems();
        if (new File(getBaseDir(), "Akut").exists()) bereich.addItem("Akut");
        if (new File(getBaseDir(), "Reha").exists()) bereich.addItem("Reha");
        if (new File(getBaseDir(), "Psy").exists()) bereich.addItem("Psy");
        if (oldselection != null) bereich.setSelectedItem(oldselection);
        else if (bereich.getItemCount() > 0) bereich.setSelectedIndex(0);
    }

    private void updateDefversions(JComboBox<String> defversion, JComboBox<String> bereich) {
        String oldselection = (String) defversion.getSelectedItem();
        defversion.removeAllItems();
        if (bereich.getSelectedItem() == null) return;
        File dir = new File(getBaseDir(), (String) bereich.getSelectedItem());
        if (!dir.exists()) return;
        List<String> defs = new ArrayList<>();
        for (File defdir : dir.listFiles()) {
            if (!defdir.isDirectory()) continue;
            defs.add(defdir.getName());
        }
        Collections.sort(defs);
        Collections.reverse(defs);
        for (String def : defs) {
            defversion.addItem(def);
        }
        if (oldselection != null) defversion.setSelectedItem(oldselection);
        else if (defversion.getItemCount() > 0) defversion.setSelectedIndex(0);
    }

    private void addField(JPanel form, int x, int y, String label, JComponent component) {
        GridBagConstraints c;

        if (label != null) {
            c = new GridBagConstraints();
            c.gridx = x;
            c.gridy = y;
            c.anchor = GridBagConstraints.WEST;
            c.insets = new Insets(3, 5, 3, 5);
            form.add(new JLabel(label), c);

            x++;
        }

        c = new GridBagConstraints();
        c.gridx = x;
        c.gridy = y;
        c.anchor = GridBagConstraints.WEST;
        c.insets = new Insets(3, 5, 3, 5);
        form.add(component, c);
    }

    private Bereich getBereich(JComboBox<String> bereich) {
        String selection = (String) bereich.getSelectedItem();

        if (selection == null) throw new IllegalStateException("Invalid bereich");

        if (selection.equals("Akut")) return Bereich.Akutsomatik;
        if (selection.equals("Reha")) return Bereich.Rehabilitation;
        if (selection.equals("Psy")) return Bereich.Psychiatrie;

        throw new IllegalStateException("Unknown bereich");
    }

    public static void main(String[] args) throws Exception {
        UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        new OnlineGrouperExample().setVisible(true);
    }

}
